//*****************************************************
//
// ExemploGLUT-texto.cpp
// Um programa OpenGL simples que exibe textos
// usando a GLUT.
//
// Marcelo Cohen e Isabel H. Manssour
// Este cdigo acompanha o livro 
// "OpenGL - Uma Abordagem Prtica e Objetiva"
// 
//*****************************************************

#include <stdio.h>
#include <stdlib.h>
#include <GL/glut.h>

// Funo que recebe a fonte e um texto por parmetro para ser exibido na 
// tela usando caracteres bitmap
void DesenhaTexto(void *font, char *string) 
{
	// Exibe caractere a caractere
	while(*string)
		glutBitmapCharacter(GLUT_BITMAP_TIMES_ROMAN_24,*string++); 
}

// Funo que recebe a fonte e um texto por parmetro para ser exibido na 
// tela usando fonte de linhas
void DesenhaTextoStroke(void *font, char *string) 
{  
	// Exibe caractere a caractere
	while(*string)
		glutStrokeCharacter(GLUT_STROKE_ROMAN,*string++); 
}

// Funo callback de redesenho da janela de visualizao
void Desenha(void)
{
	// Limpa a janela de visualizao com a cor de fundo definida
	glClear(GL_COLOR_BUFFER_BIT);

	// Define a cor para os textos: preto
	glColor3f(0,0,0);

	// Posiciona o texto stroke usando transformaes geomtricas
	glPushMatrix();	
	glTranslatef(-50,0,0);
	//glScalef(0.2, 0.2, 0.2); // diminui o tamanho do fonte
	glScalef(0.1, 0.1, 0.1); // diminui o tamanho do fonte
	//glRotatef(45, 0,0,1); // rotaciona o texto
	glRotatef(5, 0,0,1); // rotaciona o texto
	//glLineWidth(3); // define a espessura da linha
	glLineWidth(1); // define a espessura da linha
	DesenhaTextoStroke(GLUT_STROKE_ROMAN,"Stroke");
	glPopMatrix();

	// Posio no universo onde o texto bitmap ser colocado 
	glRasterPos2f(0,0); 
	DesenhaTexto(GLUT_BITMAP_TIMES_ROMAN_24,"Bitmap");
	// Exibe o desenha na janela
	glutSwapBuffers();
}

// Funo callback chamada para gerenciar eventos de teclas
void Teclado (unsigned char key, int x, int y) 
{
	if (key == 27) 
		exit(0);
}

// Inicializa projeo
void Inicializa(void)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(-100,100,-100,100);
	glMatrixMode(GL_MODELVIEW);
	
	// Define a cor de fundo de tela como branca
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
}

// Programa Principal
int main(void)
{
	// Define do modo de operao da GLUT
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_DEPTH | GLUT_RGB );

	// Especifica o tamanho inicial em pixels da janela GLUT
	glutInitWindowSize(500,400);

	// Cria a janela passando como argumento o ttulo da mesma
	glutCreateWindow("Textos com a GLUT");

	// Registra a funo callback de redesenho da janela de visualizao
	glutDisplayFunc (Desenha);

	// Registra a funo callback para tratamento das teclas ASCII
	glutKeyboardFunc (Teclado);

	// Inicializao
	Inicializa();

	// Inicia o processamento e aguarda interaes do usurio
	glutMainLoop();

	return 0;
}

