//*****************************************************
//
// ExemploTriangulos2DFan.cpp 
// Um programa OpenGL simples que abre uma janela GLUT 
// e faz o desenho de uma seqncia de tringulos 
// conectados a partir de uma mesma origem
//
// Marcelo Cohen e Isabel H. Manssour
// Este cdigo acompanha o livro
// "OpenGL - Uma Abordagem Prtica e Objetiva" 
// 
//*****************************************************

#include <stdlib.h>
#include <GL/glut.h>

// Funo callback de redesenho da janela de visualizao
void Desenha(void)
{
	// Limpa a janela de visualizao com a cor  
	// de fundo definida previamente
	glClear(GL_COLOR_BUFFER_BIT);

	// Altera a cor do desenho para cinza
	glColor3f(0.5f, 0.5f, 0.5f); 
  
	// Desenha uma seqencia de tringulos
	glBegin(GL_TRIANGLE_FAN);
		glVertex2f(  0.0f,  0.0f);
		glVertex2f(-20.0f,  0.0f);
		glVertex2f(  0.0f, 20.0f);        
		glVertex2f( 20.0f,  0.0f);
		glVertex2f(  0.0f,-20.0f);
		glVertex2f(-20.0f,  0.0f);
	glEnd();

	// Altera a cor do desenho para preta
	glColor3f(0.0f, 0.0f, 0.0f); 
  
	// Desenha uma seqencia de linhas
	glBegin(GL_LINES);
		glVertex2f(  0.0f,  0.0f);
		glVertex2f(-20.0f,  0.0f);
		glVertex2f(  0.0f,  0.0f);  
		glVertex2f(  0.0f, 20.0f);
		glVertex2f(  0.0f,  0.0f);               
		glVertex2f( 20.0f,  0.0f);
		glVertex2f(  0.0f,  0.0f);       
		glVertex2f(  0.0f,-20.0f);
	glEnd();
        
	// Executa os comandos OpenGL 
	glFlush();
}

// Funo callback chamada quando o tamanho da janela  alterado 
void AlteraTamanhoJanela(GLsizei w, GLsizei h)
{
	// Evita a divisao por zero
	if(h == 0) h = 1;
                           
	// Especifica as dimenses da Viewport
	glViewport(0, 0, w, h);

	// Inicializa o sistema de coordenadas
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// Estabelece a janela de seleo (esquerda, direita, inferior, 
	// superior) mantendo a proporo com a janela de visualizao
	if (w <= h) 
		gluOrtho2D (-50.0f, 50.0f, -50.0f*h/w, 50.0f*h/w);
	else 
		gluOrtho2D (-50.0f*w/h, 50.0f*w/h, -50.0f, 50.0f);
}

// Funo callback chamada para gerenciar eventos de teclas
void Teclado (unsigned char key, int x, int y)  
{
	if (key == 27) 
		exit(0);
}

// Funo responsvel por inicializar parmetros e variveis
void Inicializa (void)
{   
	// Define a cor de fundo da janela de visualizao como branca
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
}

// Programa Principal 
int main(void)
{
	// Define do modo de operao da GLUT
	glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB); 
 
	// Especifica a posio inicial da janela GLUT
	glutInitWindowPosition(5,5); 
    
	// Especifica o tamanho inicial em pixels da janela GLUT
	glutInitWindowSize(450,450); 
 
	// Cria a janela passando como argumento o ttulo da mesma
	glutCreateWindow("Exemplo com GL_TRIANGLE_FAN");
 
	// Registra a funo callback de redesenho da janela de visualizao
	glutDisplayFunc(Desenha);
  
	// Registra a funo callback de redimensionamento da janela de visualizao
	glutReshapeFunc(AlteraTamanhoJanela);

	// Registra a funo callback para tratamento das teclas ASCII
	glutKeyboardFunc (Teclado);

	// Chama a funo responsvel por fazer as inicializaes 
	Inicializa();
 
	// Inicia o processamento e aguarda interaes do usurio
	glutMainLoop();
 
	return 0;
}
