//*****************************************************
//
// CasaComRotTransEsc.cpp
// Um programa OpenGL simples que abre uma janela GLUT, 
// rotaciona, translada e troca a escala de um casa para
// depois desenha-la.
//
// Marcelo Cohen e Isabel H. Manssour
// Este cdigo acompanha o livro 
// "OpenGL - Uma Abordagem Prtica e Objetiva"
// 
//*****************************************************

#include <stdlib.h>
#include <GL/glut.h>

// Funo callback de redesenho da janela de visualizao
void Desenha(void)
{
	// Muda para o sistema de coordenadas do modelo
	glMatrixMode(GL_MODELVIEW);
	// Inicializa a matriz de transformao corrente
	glLoadIdentity();
     
	// Limpa a janela de visualizao com a cor  
	// de fundo definida previamente
	glClear(GL_COLOR_BUFFER_BIT);

	// Aplica uma rotao sobre a casinha que ser desenhada
	glRotatef(90.0f, 0.0f, 0.0f, 1.0f);
	// Aplica uma translao sobre a casinha que ser desenhada
	glTranslatef(15.0f, 0.0f, 0.0f);
	// Aplica uma escala sobre a casinha que ser desenhada
	glScalef(1.5f, 1.5f, 1.0f);
            
	// Desenha uma casinha composta de um quadrado e um tringulo
    
	// Altera a cor do desenho para azul
	glColor3f(0.0f, 0.0f, 1.0f);     
	// Desenha a casa
	glBegin(GL_QUADS);
		glVertex2f(-15.0f,-15.0f);
		glVertex2f(-15.0f,  5.0f);       
		glVertex2f( 15.0f,  5.0f);       
		glVertex2f( 15.0f,-15.0f);
	glEnd();

	// Altera a cor do desenho para branco
	glColor3f(1.0f, 1.0f, 1.0f);  
	// Desenha a porta e a janela  
	glBegin(GL_QUADS);
		glVertex2f(-4.0f,-14.5f);
		glVertex2f(-4.0f,  0.0f);       
		glVertex2f( 4.0f,  0.0f);       
		glVertex2f( 4.0f,-14.5f);       
		glVertex2f( 7.0f,-5.0f);
		glVertex2f( 7.0f,-1.0f);       
		glVertex2f(13.0f,-1.0f);       
		glVertex2f(13.0f,-5.0f);             
	glEnd();
    
	// Altera a cor do desenho para azul
	glColor3f(0.0f, 0.0f, 1.0f);     
	// Desenha as "linhas" da janela  
	glBegin(GL_LINES);      
		glVertex2f( 7.0f,-3.0f);      
		glVertex2f(13.0f,-3.0f);       
		glVertex2f(10.0f,-1.0f);    
		glVertex2f(10.0f,-5.0f);             
	glEnd();    
    
	// Altera a cor do desenho para vermelho
	glColor3f(1.0f, 0.0f, 0.0f); 
	// Desenha o telhado
	glBegin(GL_TRIANGLES);
		glVertex2f(-15.0f, 5.0f);   
		glVertex2f(  0.0f,17.0f);    
		glVertex2f( 15.0f, 5.0f);       
	glEnd();
        
	// Executa os comandos OpenGL 
	glFlush();
}

// Funo callback chamada quando o tamanho da janela  alterado 
void AlteraTamanhoJanela(GLsizei w, GLsizei h)
{
	GLsizei largura, altura;
                   
	// Evita a divisao por zero
	if(h == 0) h = 1;

	// Atualiza as variveis
	largura = w;
	altura = h;
                                              
	// Especifica as dimenses da Viewport
	glViewport(0, 0, largura, altura);

	// Inicializa o sistema de coordenadas
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// Estabelece a janela de seleo (esquerda, direita, inferior, 
	// superior) mantendo a proporo com a janela de visualizao
	if (largura <= altura) 
		gluOrtho2D (-40.0f, 40.0f, -40.0f*altura/largura, 40.0f*altura/largura);
	else 
		gluOrtho2D (-40.0f*largura/altura, 40.0f*largura/altura, -40.0f, 40.0f);
}

// Funo callback chamada para gerenciar eventos de teclas
void Teclado (unsigned char key, int x, int y)
{
	if (key == 27)
		exit(0);
}

// Funo responsvel por inicializar parmetros e variveis
void Inicializa (void)
{   
	// Define a cor de fundo da janela de visualizao como branca
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
}

// Programa Principal 
int main(void)
{
	// Define do modo de operao da GLUT
	glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB); 
 
	// Especifica a posio inicial da janela GLUT
	glutInitWindowPosition(5,5); 
    
	// Especifica o tamanho inicial em pixels da janela GLUT
	glutInitWindowSize(450,450); 
 
	// Cria a janela passando como argumento o ttulo da mesma
	glutCreateWindow("Desenho de uma casa com rotao, translao e escala");
 
	// Registra a funo callback de redesenho da janela de visualizao
	glutDisplayFunc(Desenha);
  
	// Registra a funo callback de redimensionamento da janela de visualizao
	glutReshapeFunc(AlteraTamanhoJanela);

	// Registra a funo callback para tratamento das teclas ASCII
	glutKeyboardFunc (Teclado);
	
	// Chama a funo responsvel por fazer as inicializaes 
	Inicializa();
 
	// Inicia o processamento e aguarda interaes do usurio
	glutMainLoop();
 
	return 0;
}
