//*****************************************************
//
// Exemplo3DComAnimacao.cpp 
// Um programa OpenGL simples que abre uma janela GLUT 
// e desenha um "teapot" que gira ao redor do eixo y 
// para exemplificar a animao com objetos 3D.
//
// Marcelo Cohen e Isabel H. Manssour
// Este cdigo acompanha o livro 
// "OpenGL - Uma Abordagem Prtica e Objetiva"
// 
//*****************************************************

#include <stdlib.h>
#include <GL/glut.h>

GLfloat fAspect;
GLfloat angulo;

// Funo callback de redesenho da janela de visualizao
void Desenha(void)
{
	// Limpa a janela de visualizao com a cor  
	// de fundo definida previamente
	glClear(GL_COLOR_BUFFER_BIT);

	// Altera a cor do desenho para preto
	glColor3f(0.0f, 0.0f, 0.0f);

	glPushMatrix();

	// Aplica uma rotao no torus
	glRotatef(angulo, 0,1,0);

	// Funo da GLUT para fazer o desenho de um "teapot" 
	// com a cor corrente
	glutWireTeapot(40.0);
	//glutWireCube(40.0);
	//glutSolidCone(40.0, 30.0,3, 3);
	//glutWireIcosahedron();
	//glutWireSphere(30, 100, 100);
	

	glPopMatrix();

	// Executa os comandos OpenGL
	glutSwapBuffers();
}

// Funo usada para especificar o volume de visualizao
void EspecificaParametrosVisualizacao(void)
{
	// Especifica sistema de coordenadas de projeo
	glMatrixMode(GL_PROJECTION);
	
	// Inicializa sistema de coordenadas de projeo
	glLoadIdentity();

	// Especifica a projeo perspectiva(angulo,aspecto,zMin,zMax)
	gluPerspective(45,fAspect,0.5,500);

	// Especifica sistema de coordenadas do modelo
	glMatrixMode(GL_MODELVIEW);
	// Inicializa sistema de coordenadas do modelo
	glLoadIdentity();

	// Especifica posio do observador e do alvo
	gluLookAt(0,60,150, 0,0,0, 0,1,0);
}

// Funo callback chamada quando o tamanho da janela  alterado 
void AlteraTamanhoJanela(GLsizei w, GLsizei h)
{
	// Para previnir uma diviso por zero
	if ( h == 0 ) h = 1;

	// Especifica as dimenses da viewport
	glViewport(0, 0, w, h);

	// Calcula a correo de aspecto
	fAspect = (GLfloat)w/(GLfloat)h;

	EspecificaParametrosVisualizacao();
}

// Funo callback chamada pela GLUT a cada intervalo de tempo
void Anima(int value)
{
	// Muda o angulo de rotao, e se chegar a 360, passa para zero
	if( ++angulo > 360.0f )
		angulo = 0.0f;

	// Redesenha o "teapot" em outra posio
	glutPostRedisplay();
	glutTimerFunc(60,Anima, 1);
}

// Funo callback chamada para gerenciar eventos de teclas
void Teclado (unsigned char key, int x, int y)
{
	if (key == 27)
		exit(0);
}

// Funo responsvel por inicializar parmetros e variveis
void Inicializa (void)
{   
	// Define a cor de fundo da janela de visualizao como branca
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
	angulo = 0.0f;
}

// Programa Principal 
int main(void)
{
	// Define do modo de operao da GLUT
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB); 

	// Especifica a posio inicial da janela GLUT
	glutInitWindowPosition(5,5); 

	// Especifica o tamanho inicial em pixels da janela GLUT
	glutInitWindowSize(450,450); 

	// Cria a janela passando como argumento o ttulo da mesma
	glutCreateWindow("Animao de um teapot 3D");

	// Registra a funo callback de redesenho da janela de visualizao
	glutDisplayFunc(Desenha);

	// Registra a funo callback de redimensionamento da janela de visualizao
	glutReshapeFunc(AlteraTamanhoJanela);

	// Registra a funo callback que ser chamada a cada intervalo de tempo
	glutTimerFunc(60, Anima, 1);

	// Registra a funo callback para tratamento das teclas ASCII
	glutKeyboardFunc (Teclado);

	// Chama a funo responsvel por fazer as inicializaes 
	Inicializa();

	// Inicia o processamento e aguarda interaes do usurio
	glutMainLoop();

	return 0;
}
